@@ -96,8 +96,6 @@ pub struct Analysis<'a, E: UserDefinedError, I: InstructionMeter> {
     pub executable: &'a dyn Executable<E, I>,
     /// Plain list of instructions as they occur in the executable
     pub instructions: Vec<ebpf::Insn>,
-    /// Syscalls used by the executable (available if debug symbols are not stripped)
-    pub syscalls: BTreeMap<u32, String>,
     /// Functions in the executable
     pub functions: BTreeMap<usize, (u32, String)>,
     /// Nodes of the control-flow graph
@@ -116,7 +114,7 @@ impl<'a, E: UserDefinedError, I: InstructionMeter> Analysis<'a, E, I> {
     /// Analyze an executable statically
     pub fn from_executable(executable: &'a dyn Executable<E, I>) -> Self {
         let (_program_vm_addr, program) = executable.get_text_bytes().unwrap();
-        let (syscalls, functions) = executable.get_symbols();
+        let functions = executable.get_function_symbols();
         debug_assert!(
             program.len() % ebpf::INSN_SIZE == 0,
             "eBPF program length must be a multiple of {:?} octets is {:?}",
@@ -140,7 +138,6 @@ impl<'a, E: UserDefinedError, I: InstructionMeter> Analysis<'a, E, I> {
         let mut result = Self {
             executable,
             instructions,
-            syscalls,
             functions,
             cfg_nodes: BTreeMap::new(),
             topological_order: Vec::new(),
@@ -164,7 +161,7 @@ impl<'a, E: UserDefinedError, I: InstructionMeter> Analysis<'a, E, I> {
             }
             for destination in &destinations {
                 self.cfg_nodes
-                    .get_mut(&destination)
+                    .get_mut(destination)
                     .unwrap()
                     .sources
                     .push(source);
@@ -189,7 +186,11 @@ impl<'a, E: UserDefinedError, I: InstructionMeter> Analysis<'a, E, I> {
             let target_pc = (insn.ptr as isize + insn.off as isize + 1) as usize;
             match insn.opc {
                 ebpf::CALL_IMM => {
-                    if let Some(syscall_name) = self.syscalls.get(&(insn.imm as u32)) {
+                    if let Some(syscall_name) = self
+                        .executable
+                        .get_syscall_symbols()
+                        .get(&(insn.imm as u32))
+                    {
                         if syscall_name == "abort" {
                             self.cfg_nodes
                                 .entry(insn.ptr + 1)
@@ -281,7 +282,7 @@ impl<'a, E: UserDefinedError, I: InstructionMeter> Analysis<'a, E, I> {
                 .filter(|(cfg_node_start, _cfg_node)| {
                     match self
                         .instructions
-                        .binary_search_by(|insn| insn.ptr.cmp(&cfg_node_start))
+                        .binary_search_by(|insn| insn.ptr.cmp(cfg_node_start))
                     {
                         Ok(_) => true,
                         Err(_index) => false,
@@ -381,7 +382,7 @@ impl<'a, E: UserDefinedError, I: InstructionMeter> Analysis<'a, E, I> {
     /// Gives the basic blocks names
     pub fn label_basic_blocks(&mut self) {
         for (pc, cfg_node) in self.cfg_nodes.iter_mut() {
-            cfg_node.label = if let Some(function) = self.functions.get(&pc) {
+            cfg_node.label = if let Some(function) = self.functions.get(pc) {
                 demangle(&function.1).to_string()
             } else {
                 format!("lbb_{}", pc)
@@ -425,7 +426,7 @@ impl<'a, E: UserDefinedError, I: InstructionMeter> Analysis<'a, E, I> {
                 insn.ptr,
                 &mut last_basic_block,
             )?;
-            writeln!(output, "    {}", disassemble_instruction(&insn, self))?;
+            writeln!(output, "    {}", disassemble_instruction(insn, self))?;
         }
         Ok(())
     }
@@ -477,7 +478,7 @@ impl<'a, E: UserDefinedError, I: InstructionMeter> Analysis<'a, E, I> {
                 cfg_node_start,
                 analysis.instructions[cfg_node.instructions.clone()].iter()
                 .map(|insn| {
-                    let desc = disassemble_instruction(&insn, &analysis);
+                    let desc = disassemble_instruction(insn, analysis);
                     if let Some(split_index) = desc.find(' ') {
                         let mut rest = desc[split_index+1..].to_string();
                         if rest.len() > MAX_CELL_CONTENT_LENGTH + 1 {
@@ -545,13 +546,13 @@ impl<'a, E: UserDefinedError, I: InstructionMeter> Analysis<'a, E, I> {
             writeln!(
                 output,
                 "    label={:?};",
-                html_escape(&self.cfg_nodes[&function_start].label)
+                html_escape(&self.cfg_nodes[function_start].label)
             )?;
             writeln!(output, "    tooltip=lbb_{};", *function_start)?;
             emit_cfg_node(
                 output,
                 dynamic_analysis,
-                &self,
+                self,
                 *function_start..function_end,
                 &mut alias_nodes,
                 *function_start,
