@@ -17,18 +17,17 @@ use solana_rbpf::{
     elf::ElfError,
     error::EbpfError,
     memory_region::AccessType,
-    syscalls,
+    syscalls::{self, Result},
     user_error::UserError,
     vm::{Config, EbpfVm, Executable, SyscallObject, SyscallRegistry},
 };
 use std::{fs::File, io::Read};
 use test_utils::{
-    BpfSyscallString, BpfSyscallU64, BpfTracePrintf, Result, SyscallWithContext,
     TestInstructionMeter, PROG_TCP_PORT_80, TCP_SACK_ASM, TCP_SACK_MATCH, TCP_SACK_NOMATCH,
 };
 
 macro_rules! test_interpreter_and_jit {
-    (register, $executable:expr, $syscall_registry:expr, $location:expr => $syscall_function:expr; $syscall_context_object:expr) => {
+    (register, $syscall_registry:expr, $location:expr => $syscall_function:expr; $syscall_context_object:expr) => {
         $syscall_registry.register_syscall_by_name::<UserError, _>($location, $syscall_function).unwrap();
     };
     (bind, $vm:expr, $location:expr => $syscall_function:expr; $syscall_context_object:expr) => {
@@ -36,9 +35,6 @@ macro_rules! test_interpreter_and_jit {
     };
     ($executable:expr, $mem:tt, ($($location:expr => $syscall_function:expr; $syscall_context_object:expr),* $(,)?), $check:block, $expected_instruction_count:expr) => {
         let check_closure = $check;
-        let mut syscall_registry = SyscallRegistry::default();
-        $(test_interpreter_and_jit!(register, $executable, syscall_registry, $location => $syscall_function; $syscall_context_object);)*
-        $executable.set_syscall_registry(syscall_registry);
         let (instruction_count_interpreter, _tracer_interpreter) = {
             let mut mem = $mem;
             let mut vm = EbpfVm::new($executable.as_ref(), &mut mem, &[]).unwrap();
@@ -87,7 +83,9 @@ macro_rules! test_interpreter_and_jit_asm {
                 enable_instruction_tracing: true,
                 ..Config::default()
             };
-            let mut executable = assemble($source, None, config).unwrap();
+            let mut syscall_registry = SyscallRegistry::default();
+            $(test_interpreter_and_jit!(register, syscall_registry, $location => $syscall_function; $syscall_context_object);)*
+            let mut executable = assemble($source, None, config, syscall_registry).unwrap();
             test_interpreter_and_jit!(executable, $mem, ($($location => $syscall_function; $syscall_context_object),*), $check, $expected_instruction_count);
         }
     };
@@ -104,7 +102,9 @@ macro_rules! test_interpreter_and_jit_elf {
                 enable_instruction_tracing: true,
                 ..Config::default()
             };
-            let mut executable = <dyn Executable::<UserError, TestInstructionMeter>>::from_elf(&elf, None, config).unwrap();
+            let mut syscall_registry = SyscallRegistry::default();
+            $(test_interpreter_and_jit!(register, syscall_registry, $location => $syscall_function; $syscall_context_object);)*
+            let mut executable = <dyn Executable::<UserError, TestInstructionMeter>>::from_elf(&elf, None, config, syscall_registry).unwrap();
             test_interpreter_and_jit!(executable, $mem, ($($location => $syscall_function; $syscall_context_object),*), $check, $expected_instruction_count);
         }
     };
@@ -2278,7 +2278,7 @@ fn test_relative_call() {
         "tests/elfs/relative_call.so",
         [1],
         (
-            b"log" => BpfSyscallString::call; BpfSyscallString {},
+            b"log" => syscalls::BpfSyscallString::call; syscalls::BpfSyscallString {},
         ),
         { |_vm, res: Result| { res.unwrap() == 2 } },
         14
@@ -2291,7 +2291,7 @@ fn test_bpf_to_bpf_scratch_registers() {
         "tests/elfs/scratch_registers.so",
         [1],
         (
-            b"log_64" => BpfSyscallU64::call; BpfSyscallU64 {},
+            b"log_64" => syscalls::BpfSyscallU64::call; syscalls::BpfSyscallU64 {},
         ),
         { |_vm, res: Result| { res.unwrap() == 112 } },
         41
@@ -2321,7 +2321,7 @@ fn test_syscall_parameter_on_stack() {
         exit",
         [],
         (
-            b"BpfSyscallString" => BpfSyscallString::call; BpfSyscallString {},
+            b"BpfSyscallString" => syscalls::BpfSyscallString::call; syscalls::BpfSyscallString {},
         ),
         { |_vm, res: Result| { res.unwrap() == 0 } },
         6
@@ -2495,7 +2495,7 @@ fn test_bpf_to_bpf_depth() {
             "tests/elfs/multiple_file.so",
             [i as u8],
             (
-                b"log" => BpfSyscallString::call; BpfSyscallString {},
+                b"log" => syscalls::BpfSyscallString::call; syscalls::BpfSyscallString {},
             ),
             { |_vm, res: Result| { res.unwrap() == 0 } },
             if i == 0 { 4 } else { 3 + 10 * i as u64 }
@@ -2510,7 +2510,7 @@ fn test_err_bpf_to_bpf_too_deep() {
         "tests/elfs/multiple_file.so",
         [config.max_call_depth as u8],
         (
-            b"log" => BpfSyscallString::call; BpfSyscallString {},
+            b"log" => syscalls::BpfSyscallString::call; syscalls::BpfSyscallString {},
         ),
         {
             |_vm, res: Result| {
@@ -2584,7 +2584,7 @@ fn test_err_syscall_string() {
         exit",
         [72, 101, 108, 108, 111],
         (
-            b"BpfSyscallString" => BpfSyscallString::call; BpfSyscallString {},
+            b"BpfSyscallString" => syscalls::BpfSyscallString::call; syscalls::BpfSyscallString {},
         ),
         {
             |_vm, res: Result| {
@@ -2608,7 +2608,7 @@ fn test_syscall_string() {
         exit",
         [72, 101, 108, 108, 111],
         (
-            b"BpfSyscallString" => BpfSyscallString::call; BpfSyscallString {},
+            b"BpfSyscallString" => syscalls::BpfSyscallString::call; syscalls::BpfSyscallString {},
         ),
         { |_vm, res: Result| { res.unwrap() == 0 } },
         4
@@ -2629,7 +2629,7 @@ fn test_syscall() {
         exit",
         [],
         (
-            b"BpfSyscallU64" => BpfSyscallU64::call; BpfSyscallU64 {},
+            b"BpfSyscallU64" => syscalls::BpfSyscallU64::call; syscalls::BpfSyscallU64 {},
         ),
         { |_vm, res: Result| { res.unwrap() == 0 } },
         8
@@ -2692,10 +2692,10 @@ fn test_syscall_with_context() {
         exit",
         [],
         (
-            b"SyscallWithContext" => SyscallWithContext::call; SyscallWithContext { context: 42 },
+            b"SyscallWithContext" => syscalls::SyscallWithContext::call; syscalls::SyscallWithContext { context: 42 },
         ),
         { |vm: &EbpfVm<UserError, TestInstructionMeter>, res: Result| {
-            let syscall_context_object = unsafe { &*(vm.get_syscall_context_object(SyscallWithContext::call as usize).unwrap() as *const SyscallWithContext) };
+            let syscall_context_object = unsafe { &*(vm.get_syscall_context_object(syscalls::SyscallWithContext::call as usize).unwrap() as *const syscalls::SyscallWithContext) };
             assert_eq!(syscall_context_object.context, 84);
             res.unwrap() == 0
         }},
@@ -2711,8 +2711,8 @@ fn test_load_elf() {
         "tests/elfs/noop.so",
         [],
         (
-            b"log" => BpfSyscallString::call; BpfSyscallString {},
-            b"log_64" => BpfSyscallU64::call; BpfSyscallU64 {},
+            b"log" => syscalls::BpfSyscallString::call; syscalls::BpfSyscallString {},
+            b"log_64" => syscalls::BpfSyscallU64::call; syscalls::BpfSyscallU64 {},
         ),
         { |_vm, res: Result| { res.unwrap() == 0 } },
         11
@@ -2725,7 +2725,7 @@ fn test_load_elf_empty_noro() {
         "tests/elfs/noro.so",
         [],
         (
-            b"log_64" => BpfSyscallU64::call; BpfSyscallU64 {},
+            b"log_64" => syscalls::BpfSyscallU64::call; syscalls::BpfSyscallU64 {},
         ),
         { |_vm, res: Result| { res.unwrap() == 0 } },
         8
@@ -2738,7 +2738,7 @@ fn test_load_elf_empty_rodata() {
         "tests/elfs/empty_rodata.so",
         [],
         (
-            b"log_64" => BpfSyscallU64::call; BpfSyscallU64 {},
+            b"log_64" => syscalls::BpfSyscallU64::call; syscalls::BpfSyscallU64 {},
         ),
         { |_vm, res: Result| { res.unwrap() == 0 } },
         8
@@ -2751,15 +2751,27 @@ fn test_custom_entrypoint() {
     let mut elf = Vec::new();
     file.read_to_end(&mut elf).unwrap();
     elf[24] = 80; // Move entrypoint to later in the text section
-    let mut executable =
-        <dyn Executable<UserError, TestInstructionMeter>>::from_elf(&elf, None, Config::default())
-            .unwrap();
+    let config = Config {
+        enable_instruction_tracing: true,
+        ..Config::default()
+    };
+    let mut syscall_registry = SyscallRegistry::default();
+    test_interpreter_and_jit!(register, syscall_registry, b"log" => syscalls::BpfSyscallString::call; syscalls::BpfSyscallString {});
+    test_interpreter_and_jit!(register, syscall_registry, b"log_64" => syscalls::BpfSyscallU64::call; syscalls::BpfSyscallU64 {});
+    #[allow(unused_mut)]
+    let mut executable = <dyn Executable<UserError, TestInstructionMeter>>::from_elf(
+        &elf,
+        None,
+        config,
+        syscall_registry,
+    )
+    .unwrap();
     test_interpreter_and_jit!(
         executable,
         [],
         (
-            b"log" => BpfSyscallString::call; BpfSyscallString {},
-            b"log_64" => BpfSyscallU64::call; BpfSyscallU64 {},
+            b"log" => syscalls::BpfSyscallString::call; syscalls::BpfSyscallString {},
+            b"log_64" => syscalls::BpfSyscallU64::call; syscalls::BpfSyscallU64 {},
         ),
         { |_vm, res: Result| { res.unwrap() == 0 } },
         2
@@ -2864,7 +2876,7 @@ fn test_instruction_count_syscall() {
         exit",
         [72, 101, 108, 108, 111],
         (
-            b"BpfSyscallString" => BpfSyscallString::call; BpfSyscallString {},
+            b"BpfSyscallString" => syscalls::BpfSyscallString::call; syscalls::BpfSyscallString {},
         ),
         { |_vm, res: Result| { res.unwrap() == 0 } },
         4
@@ -2881,7 +2893,7 @@ fn test_err_instruction_count_syscall_capped() {
         exit",
         [72, 101, 108, 108, 111],
         (
-            b"BpfSyscallString" => BpfSyscallString::call; BpfSyscallString {},
+            b"BpfSyscallString" => syscalls::BpfSyscallString::call; syscalls::BpfSyscallString {},
         ),
         {
             |_vm, res: Result| {
@@ -2939,7 +2951,7 @@ fn test_err_non_terminate_capped() {
         exit",
         [],
         (
-            b"BpfTracePrintf" => BpfTracePrintf::call; BpfTracePrintf {},
+            b"BpfTracePrintf" => syscalls::BpfTracePrintf::call; syscalls::BpfTracePrintf {},
         ),
         {
             |_vm, res: Result| {
@@ -2969,7 +2981,7 @@ fn test_err_non_terminating_capped() {
         exit",
         [],
         (
-            b"BpfTracePrintf" => BpfTracePrintf::call; BpfTracePrintf {},
+            b"BpfTracePrintf" => syscalls::BpfTracePrintf::call; syscalls::BpfTracePrintf {},
         ),
         {
             |_vm, res: Result| {
@@ -2997,7 +3009,7 @@ fn test_symbol_relocation() {
         exit",
         [72, 101, 108, 108, 111],
         (
-            b"BpfSyscallString" => BpfSyscallString::call; BpfSyscallString {},
+            b"BpfSyscallString" => syscalls::BpfSyscallString::call; syscalls::BpfSyscallString {},
         ),
         { |_vm, res: Result| { res.unwrap() == 0 } },
         6
@@ -3042,16 +3054,17 @@ fn test_err_call_unresolved() {
 
 #[test]
 fn test_err_unresolved_elf() {
-    test_interpreter_and_jit_elf!(
-        "tests/elfs/unresolved_syscall.so",
-        [],
-        (
-            b"log" => BpfSyscallString::call; BpfSyscallString {},
-        ),
-        {
-            |_vm, res: Result| matches!(res.unwrap_err(), EbpfError::ElfError(ElfError::UnresolvedSymbol(symbol, pc, offset)) if symbol == "log_64" && pc == 550 && offset == 4168)
-        },
-        9
+    let mut syscall_registry = SyscallRegistry::default();
+    test_interpreter_and_jit!(register, syscall_registry, b"log" => syscalls::BpfSyscallString::call; syscalls::BpfSyscallString {});
+    let mut file = File::open("tests/elfs/unresolved_syscall.so").unwrap();
+    let mut elf = Vec::new();
+    file.read_to_end(&mut elf).unwrap();
+    let config = Config {
+        reject_unresolved_syscalls: true,
+        ..Default::default()
+    };
+    assert!(
+        matches!(<dyn Executable::<UserError, TestInstructionMeter>>::from_elf(&elf, None, config, syscall_registry), Err(EbpfError::ElfError(ElfError::UnresolvedSymbol(symbol, pc, offset))) if symbol == "log_64" && pc == 550 && offset == 4168)
     );
 }
 
@@ -3275,20 +3288,20 @@ fn execute_generated_program(prog: &[u8]) -> bool {
     let mut bpf_functions = BTreeMap::new();
     register_bpf_function(&mut bpf_functions, 0, "entrypoint").unwrap();
     let executable = <dyn Executable<UserError, TestInstructionMeter>>::from_text_bytes(
-        &prog,
-        bpf_functions,
+        prog,
         Some(check),
         Config {
             enable_instruction_tracing: true,
             ..Config::default()
         },
+        SyscallRegistry::default(),
+        bpf_functions,
     );
     let mut executable = if let Ok(executable) = executable {
         executable
     } else {
         return false;
     };
-    executable.set_syscall_registry(SyscallRegistry::default());
     if executable.jit_compile().is_err() {
         return false;
     }
